@@ -125,6 +125,7 @@ int main(int argc, char *argv[])
   char           *acceptLanguage = (char*)"en", *lang=getenv((char*)"HTTP_ACCEPT_LANGUAGE");
   Resource       *i18nResource = NULL;
 
+  (void)_ai; /* get rid of unused variable warning */
   i18nResource = res_find_and_init_resource(PROPERTYDIR, RESOURCE_FILE);
 
   if (lang) acceptLanguage = strdup(lang);
@@ -184,10 +185,10 @@ int main(int argc, char *argv[])
     break;
   case 0:
 #if defined(WITH_SYSTEMD)
-    PL_strncpyz(line, sizeof(line), "/bin/systemctl restart " PACKAGE_NAME ".service");
+    PL_strncpyz(line, "/bin/systemctl restart " PACKAGE_NAME ".service", sizeof(line));
     restart(line);
 #elif defined(ENABLE_SERVICE)
-    PL_strncpyz(line, sizeof(line), "service " PACKAGE_NAME " restart");
+    PL_strncpyz(line, "service " PACKAGE_NAME " restart", sizeof(line));
     restart(line);
 #else
     if (util_find_file_in_paths(line,  sizeof(line), "restart-ds-admin", CMDBINDIR, "../..", "")) {
