@@ -38,6 +38,10 @@
 extern "C" {
 #endif
 
+#include <string.h>
+#include "nspr.h"
+#include "pk11func.h"
+
 #include "cert.h"
 #include "key.h"
 #include "certdb.h"
@@ -81,7 +85,7 @@ extern "C" {
 #define DBT_PSET_INV_ATTR                resource_key(RESOURCE_FILE, "16")
 #define DBT_ADMIN_CONF_MOD               resource_key(RESOURCE_FILE, "17")
 
-static int update_conf(char *configdir, char *file, char *name, char *val);
+static int update_conf(char *configdir, char *file, char *name, char *val, int quoted);
 
 Resource       *i18nResource;
 char           *acceptLanguage;
@@ -213,18 +217,46 @@ int get_cert_nickname(char *buf, size_t bufsize) {
   }
   return -1;
 }
+
+void
+drop_non_fips(char *val)
+{
+    char *p = NULL;
+    char *endp = NULL;
+    if (!val) {
+        return;
+    }
+    p = PL_strchr(val, '+');
+    while (p) {
+        endp = PL_strchr(p, ',');
+        if (endp) {
+            *endp = '\0';
+            /* E.g., p = "+rsa_rc2_40_md5" or p = "+fips_3des_sha" */
+            if (!PL_strcasestr(p, "fips")) {
+                *p = '-';
+            }
+            *endp = ',';
+            p = PL_strchr(endp+1, '+');
+        } else {
+            break;
+        }
+    }
+}
+
 /*
  * int GetSSLFamilyAttributes
  * Reads all LDAP entries relating to cipher family information.
  * Returns return_string, a string of all information found, and
  * 0 on success, -1 on failure.
  */
-int GetSSLFamilyAttributes(PsetHndl pset, char **return_string) {
-
+int
+GetSSLFamilyAttributes(PsetHndl pset, char **return_string)
+{
   AttrNameList family_list;
   int errorCode;
   char temp_return[5000];
-  char temp[1000];
+  char *tmpp = NULL;
+  size_t tmplen = 0;
 
   char **family;
   char family_attribute[1024];
@@ -233,6 +265,7 @@ int GetSSLFamilyAttributes(PsetHndl pset, char **return_string) {
   char *val;
 
   char *family_name;
+  PRBool isfips = PR_FALSE;
 
   *return_string = NULL;
   strcpy(temp_return, "");
@@ -240,11 +273,9 @@ int GetSSLFamilyAttributes(PsetHndl pset, char **return_string) {
   val = psetGetAttrSingleValue(pset,
                                "configuration.nsServerSecurity",
                                &errorCode);
-  if(val)
-    PR_snprintf(temp, sizeof(temp), "security=%s\n", val);
-  else
-    PR_snprintf(temp, sizeof(temp), "security=off\n");
-  PL_strcatn(temp_return, sizeof(temp_return), temp);
+  tmplen = strlen(temp_return);
+  tmpp = temp_return + tmplen;
+  PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "security=%s\n", val?val:"off");
 
   if((family_list = psetGetChildren(pset, "configuration.Encryption", &errorCode))) {
     
@@ -276,67 +307,75 @@ int GetSSLFamilyAttributes(PsetHndl pset, char **return_string) {
       family_name = strrchr(*family, '.');
       family_name++;
         
-      PR_snprintf(temp, sizeof(temp), "familyList=%s\n", family_name);
-      PL_strcatn(temp_return, sizeof(temp_return), temp);
+      tmplen = strlen(temp_return);
+      tmpp = temp_return + tmplen;
+      PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "familyList=%s\n", family_name);
 
-      PR_snprintf(temp, sizeof(temp), "%s-activated=%s\n", family_name, val);
-      PL_strcatn(temp_return, sizeof(temp_return), temp);
+      tmplen = strlen(temp_return);
+      tmpp = temp_return + tmplen;
+      PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "%s-activated=%s\n", family_name, val);
 
-      PR_snprintf(temp, sizeof(temp), "%s-token=%s\n", family_name, token);
-      PL_strcatn(temp_return, sizeof(temp_return), temp);
+      tmplen = strlen(temp_return);
+      tmpp = temp_return + tmplen;
+      PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "%s-token=%s\n", family_name, token);
 
-      PR_snprintf(temp, sizeof(temp), "%s-cert=%s\n", family_name, personality);
-      PL_strcatn(temp_return, sizeof(temp_return), temp);
+      tmplen = strlen(temp_return);
+      tmpp = temp_return + tmplen;
+      PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "%s-cert=%s\n", family_name, personality);
     }
   }
   PL_strcatn(temp_return, sizeof(temp_return), "familyList=NULL\n");
 
   /* get cipher preferences */
+  isfips = PK11_IsFIPS();
   
   val = NULL;
   val = psetGetAttrSingleValue(pset,
                                "configuration.encryption.nsSSL2",
                                &errorCode);
-  PL_strcatn(temp_return, sizeof(temp_return), "ssl2-activated=");
-  if(val)
-    PL_strcatn(temp_return, sizeof(temp_return), val);
-  PL_strcatn(temp_return, sizeof(temp_return), "\n");
+  tmplen = strlen(temp_return);
+  tmpp = temp_return + tmplen;
+  PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "ssl2-activated=%s\n", val?val:"");
 
   val = NULL;
   val = psetGetAttrSingleValue(pset,
                                "configuration.encryption.nsSSL2Ciphers",
                                &errorCode);
-  PL_strcatn(temp_return, sizeof(temp_return), "ssl2=");
-  if(val) 
-    PL_strcatn(temp_return, sizeof(temp_return), val);
-  PL_strcatn(temp_return, sizeof(temp_return), "\n");
+  /* If is fips, don't allow ciphers without "fips" */
+  if (isfips) {
+    drop_non_fips(val);
+  }
+  tmplen = strlen(temp_return);
+  tmpp = temp_return + tmplen;
+  PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "ssl2=%s\n", val?val:"");
 
   val = NULL;
   val = psetGetAttrSingleValue(pset,
                                "configuration.encryption.nsSSL3",
                                &errorCode);
-  PL_strcatn(temp_return, sizeof(temp_return), "ssl3-activated=");
-  if(val)
-    PL_strcatn(temp_return, sizeof(temp_return), val);
-  PL_strcatn(temp_return, sizeof(temp_return), "\n");
+  tmplen = strlen(temp_return);
+  tmpp = temp_return + tmplen;
+  PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "ssl3-activated=%s\n", val?val:"");
 
   val = NULL;
   val = psetGetAttrSingleValue(pset,
                                "configuration.encryption.nsSSL3Ciphers",
                                &errorCode);
-  PL_strcatn(temp_return, sizeof(temp_return), "ssl3=");
-  if(val)
-    PL_strcatn(temp_return, sizeof(temp_return), val);
-  PL_strcatn(temp_return, sizeof(temp_return), "\n");
+  /* If is fips, don't allow ciphers without "fips" */
+  if (isfips) {
+    drop_non_fips(val);
+  }
+  tmplen = strlen(temp_return);
+  tmpp = temp_return + tmplen;
+  PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "ssl3=%s\n", val?val:"");
 
   val = NULL;
   val = psetGetAttrSingleValue(pset,
                                "configuration.encryption.nsSSLClientAuth",
                                &errorCode);
-  PL_strcatn(temp_return, sizeof(temp_return), "clientauth=");
-  if(val)
-    PL_strcatn(temp_return, sizeof(temp_return), val);
-  PL_strcatn(temp_return, sizeof(temp_return), "\n");
+  tmplen = strlen(temp_return);
+  tmpp = temp_return + tmplen;
+  PR_snprintf(tmpp, sizeof(temp_return) - tmplen, "clientauth=%s\n", val?val:"");
   
   *return_string = PORT_Strdup(temp_return);
   return 0;
@@ -499,6 +538,7 @@ int main(int argc, char *argv[])
   AdmldapInfo ldapInfo = NULL;
   char *lang;
 
+  (void)_ai; /* get rid of unused variable warning */
   memset((void *)errp, 0, sizeof(int));
   method = getenv("REQUEST_METHOD");
 
@@ -619,7 +659,7 @@ int main(int argc, char *argv[])
 
     /* change security parameters in console.conf */
     if (strcmp(security, "off")==0) {
-      rv = update_conf(configdir, "console.conf", "NSSEngine", "off");
+      rv = update_conf(configdir, "console.conf", "NSSEngine", "off", 0);
       if (rv < 0) {
         rpt_err(APP_ERROR, NULL, getResourceString(DBT_ADMIN_CONF_MOD), NULL);
       }
@@ -658,8 +698,8 @@ int main(int argc, char *argv[])
       if (strlen(clientauth) == 0) {
         clientauth = (char*)"off";
       }
-      rv = update_conf(configdir, "console.conf", "NSSEngine", "on");
-      rv = update_conf(configdir, "console.conf", "NSSNickname", certnickname);
+      rv = update_conf(configdir, "console.conf", "NSSEngine", "on", 0);
+      rv = update_conf(configdir, "console.conf", "NSSNickname", certnickname, 1);
 
       strcpy(protocols, "");
 
@@ -669,17 +709,17 @@ int main(int argc, char *argv[])
           strcat(protocols, "SSLv3,TLSv1,");
       protocols[strlen(protocols) - 1] = '\0'; /* remove trailing comma */
 
-      rv = update_conf(configdir, "console.conf", "NSSProtocol", protocols);
+      rv = update_conf(configdir, "console.conf", "NSSProtocol", protocols, 0);
 
       snprintf(ciphers, sizeof(ciphers), "%s,%s", ssl2, merged_ssl3);
       PR_smprintf_free(merged_ssl3);
       ciphers[sizeof(ciphers)-1] = 0;
-      rv = update_conf(configdir, "console.conf", "NSSCipherSuite", ciphers);
+      rv = update_conf(configdir, "console.conf", "NSSCipherSuite", ciphers, 0);
 
       if (!strcmp(clientauth, "on"))
-        rv = update_conf(configdir, "console.conf", "NSSVerifyClient", "require");
+        rv = update_conf(configdir, "console.conf", "NSSVerifyClient", "require", 0);
       else
-        rv = update_conf(configdir, "console.conf", "NSSVerifyClient", "none");
+        rv = update_conf(configdir, "console.conf", "NSSVerifyClient", "none", 0);
 
       if (rv < 0) {
         rpt_err(APP_ERROR, NULL, getResourceString(DBT_ADMIN_CONF_MOD), NULL);
@@ -703,7 +743,7 @@ int main(int argc, char *argv[])
  * Modify any attribute in a configuration file with a name/value pair
  * If the attribute value is NULL, remove it from the file completely.
  */
-static int update_conf(char *configdir, char *file, char *name, char *val) {
+static int update_conf(char *configdir, char *file, char *name, char *val, int quoted) {
 
   FILE *f;
   int i, modified=0;
@@ -725,7 +765,11 @@ static int update_conf(char *configdir, char *file, char *name, char *val) {
   while(fgets(inbuf, sizeof(inbuf), f) != NULL) {
     if (strncasecmp(inbuf,name,strlen(name)) == 0) { /* Line starts with the attribute name */
       if(val && *val != '\0') {
-        PR_snprintf(buf, sizeof(buf), "%s \"%s\"\n", name, val);
+        if (quoted) {
+          PR_snprintf(buf, sizeof(buf), "%s \"%s\"\n", name, val);
+        } else {
+          PR_snprintf(buf, sizeof(buf), "%s %s\n", name, val);
+        }
         lines[linecnt++] = strdup(buf);
         modified=1;
       }
@@ -740,7 +784,11 @@ static int update_conf(char *configdir, char *file, char *name, char *val) {
   fclose(f);
 
   if (!modified && (val && *val != '\0')) { /* Add the attribute name/val pair*/
-    PR_snprintf(buf, sizeof(buf), "%s \"%s\"\n", name, val);
+    if (quoted) {
+      PR_snprintf(buf, sizeof(buf), "%s \"%s\"\n", name, val);
+    } else {
+      PR_snprintf(buf, sizeof(buf), "%s %s\n", name, val);
+    }
     lines[linecnt++] = strdup(buf);
   }
 
