@@ -73,11 +73,8 @@ extern "C" {
 }
 #endif
 
-#ifdef NS_DOMESTIC
-#define MAX_KEY_BITS        1024/*2048*/
-#else
-#define MAX_KEY_BITS        512/*1024*/
-#endif
+#define DEFAULT_KEY_BITS    2048
+#define MAX_KEY_BITS        4096
 
 #define SUBJECT_NEW "Certificate request"
 #define SUBJECT_OLD "Certificate renewal"
@@ -226,9 +223,14 @@ getResourceString(char *key) {
   }
 }
 
-static char * getParameter(char *key, char *keyName) {
-
-  PR_snprintf(line, sizeof(line), getResourceString(DBT_MISSING_THIS), keyName);
+static char * getParameter(char *key, char *keyName)
+{
+  if (strcmp(keyName, getResourceString(DBT_MISSING_FILE))) {
+    /* not a missing file */
+    PR_snprintf(line, sizeof(line), getResourceString(DBT_MISSING_THIS), keyName);
+  } else {
+    PR_snprintf(line, sizeof(line), getResourceString(DBT_MISSING_FILE));
+  }
 
   return get_cgi_var(key, getResourceString(DBT_PARAMETERMISSING), line);
 }
@@ -237,12 +239,12 @@ static void closeAllSecurityDB() {
   /* close all db */
   if (NSS_IsInitialized()) {
     SSL_ClearSessionCache();
-  }
-  if (NSS_Shutdown()) {
-    PR_snprintf(line, sizeof(line), getResourceString(CMN_NSS_SHUTDOWN_ERROR),
-                PR_GetError(),
-                SSL_Strerror(PR_GetError()) ? SSL_Strerror(PR_GetError()) : "unknown");
-    rpt_warning(GENERAL_FAILURE, line, NULL, NULL);
+    if (NSS_Shutdown()) {
+      PR_snprintf(line, sizeof(line), getResourceString(CMN_NSS_SHUTDOWN_ERROR),
+                  PR_GetError(),
+                  SSL_Strerror(PR_GetError()) ? SSL_Strerror(PR_GetError()) : "unknown");
+      rpt_warning(GENERAL_FAILURE, line, NULL, NULL);
+    }
   }
 }
 
@@ -259,6 +261,7 @@ static void errorRpt(int type, char* detailInfo) {
           getResourceString(DBT_ERROR_OCCURED), 
           detailInfo, 
           NULL);
+  exit(0); /* Make sure it really exits */
 }
 
 static void errorRptExtended(int type, char* detail, char* extra) {
@@ -267,6 +270,7 @@ static void errorRptExtended(int type, char* detail, char* extra) {
           getResourceString(DBT_ERROR_OCCURED), 
           detail, 
           extra);
+  exit(0); /* Make sure it really exits */
 }
 
 /* return true if all of the chars in s are valid chars for use in
@@ -886,13 +890,20 @@ static void listCert(char* tokenName) {
 
   CERTCertList *certList;
   CERTCertListNode *cln;
-  PK11SlotInfo *slot = PK11_FindSlotByName(tokenName);
+  PK11SlotInfo *slot = NULL;
   PK11SlotInfo *internal_slot;
   char *internalTokenName;
 
+  if (tokenName && (!strcasecmp(tokenName, "internal") ||
+                    !strcasecmp(tokenName, "internal (software)"))) {
+      slot = PK11_GetInternalKeySlot();
+  } else {
+      slot = PK11_FindSlotByName(tokenName);
+  }
+
   if (!slot) {
-	  errorRpt(GENERAL_FAILURE, getResourceString(DBT_TOKEN_NAME));
-	  return;
+      errorRpt(GENERAL_FAILURE, getResourceString(DBT_TOKEN_NAME));
+      return;
   }
 
   if (PK11_IsInternal(slot)) {
@@ -1040,6 +1051,12 @@ static void securityInitialization(char* securitydir) {
 
   /* Set certdb handle */
   certdb = CERT_GetDefaultCertDB();
+  if (!certdb) {
+    PR_snprintf(line, sizeof(line), getResourceString(CMN_SSL_INIT_ERROR),
+                securitydir ? securitydir : "(null)", PR_GetError(),
+                SSL_Strerror(PR_GetError()) ? SSL_Strerror(PR_GetError()) : "unknown");
+    errorRpt(GENERAL_FAILURE, line);
+  }
 
   /* set password function */
   PK11_SetPasswordFunc(getPassword_cb);
@@ -1064,6 +1081,8 @@ generateCertificateRequest(SECKEYPrivateKey* privateKey, SECKEYPublicKey* pubKey
   PRArenaPool *arena                = NULL;
   PRBool      error                 = PR_FALSE;
   char *line;
+  char *sSignAlgo                   = NULL;
+  int signAlgo                      = 0;
   /*DebugBreak();*/
   /* convert subject name(DN) */
   certName = CERT_AsciiToName(subjectName);
@@ -1101,8 +1120,25 @@ generateCertificateRequest(SECKEYPrivateKey* privateKey, SECKEYPublicKey* pubKey
   /* Encode the result will get a "request blob" */
   der = (SECItem *)SEC_ASN1EncodeItem(arena, result, request, SEC_ASN1_GET(CERT_CertificateRequestTemplate));
 
+  /* Determine the signing algorithm to use.  We default
+  * to SHA-1 and support SHA-256, SHA-384, and SHA-512. */
+  sSignAlgo = get_cgi_var("signingalgo", NULL, NULL);
+
+  if (!sSignAlgo || !PORT_Strcmp(sSignAlgo, "SHA-1")) {
+    signAlgo = SEC_OID_PKCS1_SHA1_WITH_RSA_ENCRYPTION;
+  } else if (!PORT_Strcmp(sSignAlgo, "SHA-256")) {
+    signAlgo = SEC_OID_PKCS1_SHA256_WITH_RSA_ENCRYPTION;
+  } else if (!PORT_Strcmp(sSignAlgo, "SHA-384")) {
+    signAlgo = SEC_OID_PKCS1_SHA384_WITH_RSA_ENCRYPTION;
+  } else if (!PORT_Strcmp(sSignAlgo, "SHA-512")) {
+    signAlgo = SEC_OID_PKCS1_SHA512_WITH_RSA_ENCRYPTION;
+  } else {
+    /* Unknown algorithm, so just use the default. */
+    signAlgo = SEC_OID_PKCS1_SHA1_WITH_RSA_ENCRYPTION;
+  }
+
   /* Sign certificate request(the blob) with private key */
-  if (SEC_DerSignData(arena, result, der->data, der->len, privateKey, SEC_OID_PKCS1_MD5_WITH_RSA_ENCRYPTION) != SECSuccess)  {
+  if (SEC_DerSignData(arena, result, der->data, der->len, privateKey, signAlgo) != SECSuccess)  {
     rpt_err(GENERAL_FAILURE, 
             getResourceString(DBT_INTERNAL_ERROR), 
             getResourceString(DBT_CSR_GEN_FAIL), 
@@ -1172,16 +1208,16 @@ generateKey(SECKEYPublicKey** publicKey, char* tokenName)
 
   /* generate key pair */
   {
-
     char *sKeySize = get_cgi_var("keysize", NULL, NULL);
     int keySize = 0;
     if (sKeySize) {
       keySize = atoi(sKeySize);
     }
 
-
-    if ((keySize > MAX_KEY_BITS) || (keySize <=0)) {
+    if (keySize > MAX_KEY_BITS) {
       params.keySizeInBits = MAX_KEY_BITS;
+    } else if (keySize <= 0) {
+      params.keySizeInBits = DEFAULT_KEY_BITS;
     } else {
       params.keySizeInBits = keySize;
     }
@@ -1194,12 +1230,7 @@ generateKey(SECKEYPublicKey** publicKey, char* tokenName)
 
  loser:
   if (privateKey==NULL) {
-    char *tmpLine = NULL;
-
-    tmpLine = (char *)PR_Malloc(PR_GetErrorTextLength()+1);
-    PR_GetErrorText(tmpLine);
-    PR_snprintf(line, sizeof(line), "%d:%s", PR_GetError(), tmpLine);
-    PR_Free(tmpLine);
+    PR_snprintf(line, sizeof(line), "%d:%s", PR_GetError(), PR_ErrorToString(PR_GetError(), PR_LANGUAGE_EN));
 
     rpt_err(GENERAL_FAILURE, 
             getResourceString(DBT_INTERNAL_ERROR), 
@@ -1324,7 +1355,7 @@ static CERTDERCerts* decodeDERCert(char *derCertBase64) {
  * Decode and display a DER certificate.
  */
 static void printDERCert(int isCACert) {
-
+  SECStatus rv;
   char *derCertBase64 = getParameter("dercert",getResourceString(DBT_DER_CERT));
   CERTDERCerts *collectArgs = decodeDERCert(derCertBase64);
 
@@ -1341,12 +1372,23 @@ static void printDERCert(int isCACert) {
     char *nickname = NULL;
 
     /*add all cert to temp */
-    CERT_ImportCerts(certdb, certUsageSSLServer,
+    rv = CERT_ImportCerts(certdb, certUsageSSLServer,
                      collectArgs->numcerts,  &collectArgs->rawCerts,
                      &retCerts, keepCerts,
                      caOnly, nickname);
 
-    printCert(retCerts[collectArgs->numcerts-1], /*showDetail=*/PR_TRUE, certType);
+    if (rv == SECSuccess) {
+      printCert(retCerts[collectArgs->numcerts-1], /*showDetail=*/PR_TRUE, certType);
+    } else {
+      PR_snprintf(line, sizeof(line), "%d:%s", PR_GetError(),
+                  PR_ErrorToString(PR_GetError(), PR_LANGUAGE_EN));
+
+      /* if unable to import report error */
+      rpt_err(SYSTEM_ERROR,
+              getResourceString(DBT_INTERNAL_ERROR),
+              getResourceString(DBT_INSTALL_FAIL),
+              line);
+    }
   }
 }
 
@@ -1378,11 +1420,15 @@ installServerCert(char *tokenName, char *certname)
     PRBool caOnly = PR_FALSE;
     char *nickname = certname;
 
-    CERT_ImportCerts(certdb, certUsageSSLServer,
+    rv = CERT_ImportCerts(certdb, certUsageSSLServer,
                      ncerts,  &collectArgs->rawCerts,
                      &retCerts, keepCerts,
                      caOnly, nickname);
 
+    if (rv != SECSuccess) {
+        goto bail;
+    }
+
     cert = retCerts[0];
   }
 
@@ -1420,19 +1466,15 @@ installServerCert(char *tokenName, char *certname)
   /* import certificate to the PKCS11 module */
   rv  = PK11_ImportCertForKeyToSlot(slot, cert, certname, PR_TRUE, 0);
 
+bail:
   if (rv != SECSuccess) {
-    {
-      char *tmpLine;
+    PR_snprintf(line, sizeof(line), "%d:%s", PR_GetError(),
+                PR_ErrorToString(PR_GetError(), PR_LANGUAGE_EN));
 
-      tmpLine = (char *)PR_Malloc(PR_GetErrorTextLength()+1);
-      PR_GetErrorText(tmpLine);
-      PR_snprintf(line, sizeof(line), "%d:%s", PR_GetError(), tmpLine);
-      PR_Free(tmpLine);
-    }
     /* if unable to import report error */
-    rpt_err(SYSTEM_ERROR, 
-            getResourceString(DBT_INTERNAL_ERROR), 
-            getResourceString(DBT_INSTALL_FAIL), 
+    rpt_err(SYSTEM_ERROR,
+            getResourceString(DBT_INTERNAL_ERROR),
+            getResourceString(DBT_INSTALL_FAIL),
             line);
   }
 }
@@ -1482,14 +1524,20 @@ installCACert(char *tokenName, char *certname)
     rc = CERT_ImportCerts(certdb, (trustedCA ? certUsageSSLCA : certUsageAnyCA),
                      collectArgs->numcerts,  &collectArgs->rawCerts,
                      &retCerts, keepCerts, caOnly, nickname);
+
+    if (rc != SECSuccess) {
+        goto bail;
+    }
+
     CERT_FindCertByDERCert(certdb, collectArgs->rawCerts);
     cert = retCerts[0];
     rc  = PK11_ImportCert(slot, cert, CK_INVALID_HANDLE, certname, PR_FALSE);
+
+bail:
     if (rc != SECSuccess) {
-        char *tmpLine = (char *)PR_Malloc(PR_GetErrorTextLength()+1);
-        PR_GetErrorText(tmpLine);
-        PR_snprintf(line, sizeof(line), "%d:%s", PR_GetError(), tmpLine);
-        PR_Free(tmpLine);
+        PR_snprintf(line, sizeof(line), "%d:%s", PR_GetError(),
+                    PR_ErrorToString(PR_GetError(), PR_LANGUAGE_EN));
+
         /* if unable to import report error */
         rpt_err(SYSTEM_ERROR, getResourceString(DBT_INTERNAL_ERROR), 
                 getResourceString(DBT_INSTALL_FAIL), line);
@@ -1964,10 +2012,16 @@ static void moduleOperation(char* op) {
               install_dir,
               securitydir);
     else if (!PORT_Strcmp(filetype, "dll"))
-      PR_snprintf(cmd, sizeof(cmd), "%s -dbdir %s -add \"%s\" -libfile %s -force -nocertdb 2>&1",
+      /*
+       * Since console requires new modules to be located in the security dir,
+       * and be presented as just a filename(no path), we must append the
+       * securitydir to the filename as modutil needs the absolute path.
+       */
+      PR_snprintf(cmd, sizeof(cmd), "%s -dbdir %s -add \"%s\" -libfile %s/%s -force -nocertdb 2>&1",
               binary,
               securitydir,
               dllname,
+              securitydir,
               filename);
 
     else {
@@ -2109,7 +2163,7 @@ int main(int argc, char *argv[])
 
     if (!operation) {
       /* if we got here it means front end pass back
-	 a bogus operation string */
+         a bogus operation string */
       PR_snprintf(line, sizeof(line), getResourceString(DBT_NO_OP), operation);
       errorRpt(INCORRECT_USAGE, line);
     } else if (!PORT_Strcmp(operation, "MIGRATE_DB")) {
@@ -2184,8 +2238,15 @@ int main(int argc, char *argv[])
 
       } else if (!PORT_Strcmp(operation, "INIT_PIN")) {
         /* initialize internal token pin */
-        initPin(getParameter("newpwd",getResourceString(DBT_PASSWORD)),
-                getParameter("confirmpwd",getResourceString(DBT_PASSWORD)));
+
+        /* We fetch the paramters here for the benefit of the Console.  If we call
+         * getParameter() as an argument to initPin, the calling order causes the
+         * error message that is returned to be out of order with the display in
+         * the UI. */
+        char *newpwd = getParameter("newpwd",getResourceString(DBT_PASSWORD));
+        char *confirmpwd = getParameter("confirmpwd",getResourceString(DBT_PASSWORD));
+
+        initPin(newpwd, confirmpwd);
 
       } else if (!PORT_Strcmp(operation, "DELETE_CACERT")) {
         /* remove a ca certificate */
@@ -2218,9 +2279,16 @@ int main(int argc, char *argv[])
         /* currently change password only apply to key3.db, and we do
            not deal with change password for external token.  User can
            do that via the software that comes with the hardware */
-        changePassword(getParameter("oldpwd", getResourceString(DBT_OLD_PWD)),
-                       getParameter("newpwd", getResourceString(DBT_NEW_PWD)),
-                       getParameter("confirmpwd", getResourceString(DBT_CONFIRM_PWD)));
+
+        /* We fetch the paramters here for the benefit of the Console.  If we call
+         * getParameter() as an argument to initPin, the calling order causes the
+         * error message that is returned to be out of order with the display in
+         * the UI. */
+        char *oldpwd = getParameter("oldpwd", getResourceString(DBT_OLD_PWD));
+        char *newpwd = getParameter("newpwd", getResourceString(DBT_NEW_PWD));
+        char *confirmpwd =  getParameter("confirmpwd", getResourceString(DBT_CONFIRM_PWD));
+
+        changePassword(oldpwd, newpwd, confirmpwd);
 
       } else if (!PORT_Strcmp(operation, "INSTALL_CRL_CKL")) {
         /* install a crl/ckl certificate */
