@@ -1558,17 +1558,19 @@ util_ldap_init(
         if (secure > 0) {
 #if defined(USE_OPENLDAP)
             int optval = 0;
+#else
+            LDAP *myld = NULL;
 #endif /* !USE_OPENLDAP */
             int ssl_strength = 0;
-            LDAP *myld = NULL;
 
+#if !defined(USE_OPENLDAP)
             /* we can only use the set functions below with a real
                LDAP* if it has already gone through ldapssl_init -
                so, use NULL if using starttls */
             if (secure == 1) {
                 myld = ld;
             }
-
+#endif
             /* verify certificate only */
 #if defined(USE_OPENLDAP)
             ssl_strength = LDAP_OPT_X_TLS_NEVER;
@@ -1682,55 +1684,13 @@ util_ldap_bind(
 )
 {
     int rc = LDAP_SUCCESS;
-    int secure = 0;
+    int err = LDAP_SUCCESS;
     struct berval bvcreds = {0, NULL};
     LDAPMessage *result = NULL;
     struct berval *servercredp = NULL;
-#if defined(USE_OPENLDAP)
-    /* openldap doesn't have a SSL/TLS yes/no flag - so grab the
-       ldapurl, parse it, and see if it is a secure one */
-    char *ldapurl = NULL;
 
-    ldap_get_option(ld, LDAP_OPT_URI, &ldapurl);
-    if (ldapurl && !PL_strncasecmp(ldapurl, "ldaps", 5)) {
-        secure = 1;
-    }
-    PL_strfree(ldapurl);
-    ldapurl = NULL;
-#else /* !USE_OPENLDAP */
-    ldap_get_option(ld, LDAP_OPT_SSL, &secure);
-#endif
-
-#ifdef EXTERNAL_AUTH_SUPPORTED
-    if (secure && mech && !strcmp(mech, LDAP_SASL_EXTERNAL)) {
-        /* SSL connections will use the server's security context
-           and cert for client auth */
-        rc = slapd_SSL_client_auth(ld);
-
-        if (rc != 0) {
-#ifdef DEBUG
-            fprintf(stderr, "util_ldap_bind: "
-                    "Error: could not configure the server for cert "
-                    "auth - error %d - make sure the server is "
-                    "correctly configured for SSL/TLS\n", rc);
-#endif
-            goto done;
-        } else {
-#ifdef DEBUG
-            fprintf(stderr, "util_ldap_bind: "
-                    "Set up conn to use client auth\n");
-#endif
-        }
-        bvcreds.bv_val = NULL; /* ignore username and passed in creds */
-        bvcreds.bv_len = 0; /* for external auth */
-        bindid = NULL;
-    } else { /* other type of auth */
-#endif /* EXTERNAL_AUTH_SUPPORTED */
-        bvcreds.bv_val = (char *)creds;
-        bvcreds.bv_len = creds ? strlen(creds) : 0;
-#ifdef EXTERNAL_AUTH_SUPPORTED
-    }
-#endif /* EXTERNAL_AUTH_SUPPORTED */
+    bvcreds.bv_val = (char *)creds;
+    bvcreds.bv_len = creds ? strlen(creds) : 0;
 
     /* The connection has been set up - now do the actual bind, depending on
        the mechanism and arguments */
@@ -1788,20 +1748,30 @@ util_ldap_bind(
             }
             /* if we got here, we were able to read success result */
             /* Get the controls sent by the server if requested */
-            if (returnedctrls) {
-                if ((rc = ldap_parse_result(ld, result, &rc, NULL, NULL,
-                                            NULL, returnedctrls,
-                                            0)) != LDAP_SUCCESS) {
+            if ((rc = ldap_parse_result(ld, result, &err, NULL, NULL,
+                                        NULL, returnedctrls, 0)) != LDAP_SUCCESS) {
 #ifdef DEBUG
-                    fprintf(stderr, "util_ldap_bind: "
-                            "Error: could not bind id "
-                            "[%s] mech [%s]: error %d (%s)\n",
-                            bindid ? bindid : "(anon)",
-                            mech ? mech : "SIMPLE",
-                            rc, ldap_err2string(rc));
+                fprintf(stderr, "util_ldap_bind: "
+                        "Error: could not parse bind result "
+                        "[%s] mech [%s]: error %d (%s)\n",
+                        bindid ? bindid : "(anon)",
+                        mech ? mech : "SIMPLE",
+                        rc, ldap_err2string(rc));
 #endif
-                    goto done;
-                }
+                goto done;
+            }
+
+            if(err){
+                rc = err;
+#ifdef DEBUG
+                fprintf(stderr, "util_ldap_bind: "
+                        "Error: could not bind id "
+                        "[%s] mech [%s]: error %d (%s)\n",
+                        bindid ? bindid : "(anon)",
+                        mech ? mech : "SIMPLE",
+                        rc, ldap_err2string(rc));
+#endif
+                goto done;
             }
 
             /* parse the bind result and get the ldap error code */
